/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: main.c,v 1.19 2000/11/17 17:29:35 f1rmb Exp $
 *
 * xine main
 *
 */

/* Requiered for getsubopt() */
#define _XOPEN_SOURCE 500

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include "demux.h"
#include "xine.h"
#include "configfile.h"
#include "video_out/video_out.h"
#include "audio_out/audio_out.h"
#include "gui/gui_main.h"
#include "gui/gui_dnd.h"
#ifdef HAVE_GETOPT_LONG
#  include <getopt.h>
#else
#  include "getopt.h"
#endif

extern Display        *gDisplay;
extern vo_driver_t    *gVideoDriver;
extern Window          gVideoWin;

/* playlist */
int                    nPlayNum;
int                    ncur;
char                  *playlist[100];

/* options args */
static const char *short_options = "?h"
#ifdef HAVE_XV
 "s"
#endif
 "a:A:";
static struct option long_options[] = {
  {"help"           , no_argument      , 0, 'h' },
#ifdef HAVE_XV
  {"shared-memory"  , no_argument      , 0, 's' },
#endif
  {"audio-channel"  , required_argument, 0, 'a' },
  {"audio-driver"   , required_argument, 0, 'A' },
  {0                , no_argument      , 0,  0  }
};

extern const char* ao_available[];
int ao_use_driver = AO_UNSET;

/* ------------------------------------------------------------------ */
/*
 * Clean a string (remove spaces and '=' at the begin,
 * and '\n', '\r' and spaces at the end.
 */
char *atoa(char *str) {
  char *pbuf;

  pbuf = str;

  while(*pbuf != '\0') pbuf++;

  while(pbuf > str) {
    if(*pbuf == '\r' || *pbuf == '\n' || *pbuf == '"') pbuf = '\0';
    pbuf--;
  }

  while(*pbuf == '=' || *pbuf == '"') pbuf++;

  return pbuf;
}
/* ------------------------------------------------------------------------- */
/*
 *
 */
void show_banner(void) {

  printf("This is %s - a mpeg 1, 2 player v%s - (c) 2000 by G. Bartsch.\n", 
	 PACKAGE, VERSION);
}
/* ------------------------------------------------------------------------- */
/*
 *
 */
void show_usage (void) {

  printf("\n");
  printf("Usage: %s [OPTIONS]... [MRL]\n", PACKAGE);
  printf("\n");
  printf("OPTIONS are:\n");
#ifdef HAVE_XV
  printf("  -s, --shared-memory          Use X shared memory extension "
	 "for output.\n");
#endif
  printf("  -a, --audio-channel <#>      Select audio channel '#'.\n");
  printf("  -A, --audio-driver <drv>     Select audio driver (oss"
#ifdef HAVE_ALSA
	 ", alsa"
#endif
	 " or null).\n");
  printf("\n");
  printf("valid MRLs (media resource locator) are:\n");
  printf("  File:  'path/foo.vob'\n");
  printf("         '/path/foo.vob'\n");
  printf("         'file://path/foo.vob'\n");
  printf("         'fifo://path/foo'\n");
  printf("         'stdin:// or '-''\n");
  printf("  DVD:   'dvd://VTS_01_2.VOB'\n");
  printf("  VCD:   'vcd://<track number>'\n");
  printf("\n");
}
/* ------------------------------------------------------------------------- */
/*
 * Handle audio output multi-args, return first audio plugin load successed.
 * Return AO_UNSET on fail. 
 */
int handle_audioout_subopt(char *sopt) {
  int subopt;
  char *str = sopt;
  char *val = NULL;
  ao_functions_t *aout;
 
  while((subopt = getsubopt(&str, ao_available, &val)) != -1) {
    
    switch(subopt) {
    case AO_OSS:
      if((aout = audio_ossout_init()) != NULL) {
      	return AO_OSS;
      }
      break;

    case AO_NULL:
      if((aout = audio_nullout_init()) != NULL) {
      	return AO_NULL;
      }
      break;

#ifdef HAVE_ALSA      
    case AO_ALSA:
      if((aout = audio_alsaout_init()) != NULL) {
	return AO_ALSA;
      }
      break;
#endif

    }
  }

  return AO_UNSET;
}
/* ------------------------------------------------------------------------- */
/*
 *
 */
int main(int argc, char *argv[]) {
  int   option_index = 0;
  int   no_xv;
  int   audio_track=0;
  char  c;
  char *filename = NULL;
  char *display_name = ":0.0";

#ifdef HAVE_XV
  no_xv = 0;
#else
  no_xv = 1;
#endif

  show_banner();

  /*
   * parse command line
   */
  opterr = 0;
  while((c = getopt_long(argc, argv, short_options, 
			 long_options, &option_index)) != EOF) {
    switch(c) {

#ifdef HAVE_XV
    case 's': /* Use Xshm insteat of Xv (if Xv is present) */
      no_xv = 1;
      break;
#endif
      
    case 'a': /* Select audio channel */
      sscanf(optarg, "%i", &audio_track);
      /* track_code = track_table[audio_track]; */
      break;
      
    case 'A': /* Select audio driver */
      if(optarg != NULL) {
	if((ao_use_driver = 
	    handle_audioout_subopt(atoa(optarg))) == AO_UNSET) {
	  int i;
	  fprintf(stderr, "Error: available audio output drivers are:");
	  for(i = AO_OSS; ao_available[i] != NULL; i++)
	    fprintf(stderr, " %s,", ao_available[i]);
	  fprintf(stderr, "\b.\n");
	  exit(1);
	}
      }
      break;

    case 'h': /* Display usage */
    case '?':
      show_usage();
      exit(1);
      break;
      
    default:
      show_usage();
      printf ("invalid argument %d => exit\n",c);
      exit(1);
    }
  }

  /* If not audio output driver selected, force to OSS */
  if(ao_use_driver == AO_UNSET)
    ao_use_driver = AO_OSS;
  fprintf(stderr, "Using %s audio output driver.\n", 
	  ao_available[ao_use_driver]);

  /* Extract filename (if specified) */
  if (optind < argc) 
    filename = argv[optind];

  config_file_init ();

  /*
   * init X11
   */

  XInitThreads ();

  if(getenv("DISPLAY"))
    display_name = getenv("DISPLAY");

  gDisplay = XOpenDisplay(display_name);

  if (gDisplay == NULL) {
    fprintf(stderr,"Can not open display\n");
    exit(1);
  }

  /*
   * drag and drop
   */

  gui_init_dnd();
  gui_dnd_set_callback (gui_dndcallback);

  /*
   * xine init
   */

  xine_init (no_xv, gui_status_callback);
  xine_select_audio_channel (audio_track);

  /*
   * hand control over to gui
   */

  gui_start(argc-optind, &argv[optind]);

  return 0;
}		

